@@ -9,12 +9,17 @@
 
 """Misc functions needed in various places in Sutekh."""
 
-from sutekh.core.SutekhObjects import VersionTable, flush_cache
-from sutekh.core.DatabaseVersion import DatabaseVersion
-from sutekh.io.WhiteWolfParser import WhiteWolfParser
-from sutekh.io.RulingParser import RulingParser
+import tempfile
+import os
+import sys
+import re
 from sqlobject import sqlhub
-import tempfile, os, sys
+from sutekh.core.SutekhObjects import VersionTable, flush_cache, CRYPT_TYPES, \
+        PhysicalCardSet, canonical_to_csv
+from sutekh.core.DatabaseVersion import DatabaseVersion
+from sutekh.io.WhiteWolfTextParser import WhiteWolfTextParser
+from sutekh.io.RulingParser import RulingParser
+
 
 def refresh_tables(aTables, oConn):
     """Drop and recreate the given list of tables"""
@@ -36,6 +41,7 @@ def refresh_tables(aTables, oConn):
     flush_cache()
     return True
 
+
 def read_white_wolf_list(aWwFiles, oLogHandler=None):
     """Parse in a new White Wolf cardlist
 
@@ -45,16 +51,16 @@ def read_white_wolf_list(aWwFiles, oLogHandler=None):
     flush_cache()
     oOldConn = sqlhub.processConnection
     sqlhub.processConnection = oOldConn.transaction()
-    oParser = WhiteWolfParser(oLogHandler)
+    oParser = WhiteWolfTextParser(oLogHandler)
     for oFile in aWwFiles:
         fIn = oFile.open()
-        for sLine in fIn:
-            oParser.feed(sLine)
+        oParser.parse(fIn)
         fIn.close()
     sqlhub.processConnection.commit(close=True)
     sqlhub.processConnection = oOldConn
 
-def read_rulings(oRulings, oLogHandler=None):
+
+def read_rulings(aRulings, oLogHandler=None):
     """Parse a new White Wolf rulings file
 
        oRulings is an object with a .open() method (e.g. a sutekh.io.WwFile.WwFile)
@@ -63,13 +69,15 @@ def read_rulings(oRulings, oLogHandler=None):
     oOldConn = sqlhub.processConnection
     sqlhub.processConnection = oOldConn.transaction()
     oParser = RulingParser(oLogHandler)
-    fIn = oRulings.open()
-    for sLine in fIn:
-        oParser.feed(sLine)
-    fIn.close()
+    for oFile in aRulings:
+        fIn = oFile.open()
+        for sLine in fIn:
+            oParser.feed(sLine)
+        fIn.close()
     sqlhub.processConnection.commit(close=True)
     sqlhub.processConnection = oOldConn
 
+
 def gen_temp_file(sBaseName, sDir):
     """Simple wrapper around tempfile creation - generates the name and closes
        the file
@@ -85,22 +93,25 @@ def gen_temp_file(sBaseName, sDir):
     # I don't see it being triggered accidently
     return sFilename
 
+
 def gen_temp_dir():
     """Create a temporary directory using mkdtemp"""
     sTempdir = tempfile.mkdtemp('dir', 'sutekh')
     return sTempdir
 
+
 def safe_filename(sFilename):
     """Replace potentially dangerous and annoying characters in the name -
        used to automatically generate sensible filenames from card set names
        """
     sSafeName = sFilename
-    sSafeName = sSafeName.replace(" ", "_") # I dislike spaces in filenames
+    sSafeName = sSafeName.replace(" ", "_")  # I dislike spaces in filenames
     # Prevented unexpected filesystem issues
     sSafeName = sSafeName.replace("/", "_")
-    sSafeName = sSafeName.replace("\\", "_") # ditto for windows
+    sSafeName = sSafeName.replace("\\", "_")  # ditto for windows
     return sSafeName
 
+
 def prefs_dir(sApp):
     """Return a suitable directory for storing preferences and other
        application data."""
@@ -109,6 +120,7 @@ def prefs_dir(sApp):
     else:
         return os.path.join(os.path.expanduser("~"), ".%s" % sApp.lower())
 
+
 def ensure_dir_exists(sDir):
     """Check that a directory exists and create it if it doesn't.
        """
@@ -117,6 +129,7 @@ def ensure_dir_exists(sDir):
     else:
         os.makedirs(sDir)
 
+
 def sqlite_uri(sPath):
     """Create an SQLite db URI from the path to the db file.
        """
@@ -130,6 +143,7 @@ def sqlite_uri(sPath):
 
     return "sqlite://" + sDbFile
 
+
 def pretty_xml(oElement, iIndentLevel=0):
     """
     Helper function to add whitespace text attributes to a ElementTree.
@@ -152,3 +166,76 @@ def pretty_xml(oElement, iIndentLevel=0):
             oElement.tail = sIndent
 
 
+def format_text(sCardText):
+    """Ensure card text is formatted properly"""
+    # We want to split the . [dis] pattern into .\n[dis] again
+    sResult = re.sub('(\.|\.\)) (\[...\])', '\\1\n\\2', sCardText)
+    # But don't split the 'is not a discpline'
+    return re.sub('\n(\[...\] is not a Dis)', ' \\1', sResult)
+
+
+# Utility test for crypt cards
+def is_crypt_card(oAbsCard):
+    """Test if a card is a crypt card or not"""
+    # Vampires and Imbued have exactly one card type (we hope that WW
+    # don't change that)
+    return oAbsCard.cardtype[0].name in CRYPT_TYPES
+
+
+# Utility function to help with config management and such
+def get_cs_id_name_table():
+    """Returns a dictionary id : name for all the card sets.
+
+       We do this so we can have the old info available to fix the config
+       after a database reload, etc."""
+    dMapping = {}
+    for oCS in PhysicalCardSet.select():
+        dMapping[oCS.id] = oCS.name
+    return dMapping
+
+
+# Helper functions for the io routines
+def monger_url(oCard, bVamp):
+    """Return a monger url for the given AbstractCard"""
+    sName = canonical_to_csv(oCard.name)
+    if bVamp:
+        if oCard.level is not None:
+            sName = sName.replace(' (Advanced)', '')
+            sMongerURL = "http://monger.vekn.org/showvamp.html?NAME=%s ADV" \
+                    % sName
+        else:
+            sMongerURL = "http://monger.vekn.org/showvamp.html?NAME=%s" % sName
+    else:
+        sMongerURL = "http://monger.vekn.org/showcard.html?NAME=%s" % sName
+    # May not need this, but play safe
+    sMongerURL = sMongerURL.replace(' ', '%20')
+    return sMongerURL
+
+
+def secret_library_url(oCard, bVamp):
+    """Return a Secret Library url for the given AbstractCard"""
+    sName = canonical_to_csv(oCard.name)
+    if bVamp:
+        if oCard.level is not None:
+            sName = sName.replace(' (Advanced)', '')
+            sURL = "http://www.secretlibrary.info/?crypt=%s+Adv" \
+                    % sName
+        else:
+            sURL = "http://www.secretlibrary.info/?crypt=%s" \
+                    % sName
+    else:
+        sURL = "http://www.secretlibrary.info/?lib=%s" \
+                    % sName
+    sURL = sURL.replace(' ', '+')
+    # ET will replace " with &quot;, which can lead to issues with SL, so we
+    # drop double quotes entirely
+    sURL = sURL.replace('"', '')
+    return sURL
+
+
+def norm_xml_quotes(sData):
+    """Normalise quote escaping from ElementTree, to hide version
+       differences"""
+    # Bescause of how ElementTree adds quotes internally, this should always
+    # be safe
+    return sData.replace('&apos;', "'")
