@@ -6,6 +6,7 @@
 """This handles the gui aspects of upgrading the database."""
 
 import gtk
+import logging
 from sqlobject import sqlhub, connectionForURI
 from sutekh.gui.DBUpgradeDialog import DBUpgradeDialog
 from sutekh.gui.WWFilesDialog import WWFilesDialog
@@ -13,13 +14,14 @@ from sutekh.gui.ProgressDialog import ProgressDialog, SutekhHTMLLogHandler, \
         SutekhCountLogHandler
 from sutekh.core.DatabaseUpgrade import create_memory_copy, \
         create_final_copy, UnknownVersion, copy_to_new_abstract_card_db
-from sutekh.gui.SutekhDialog import do_complaint_buttons, do_complaint_error, \
-        do_complaint, do_complaint_warning
+from sutekh.gui.SutekhDialog import do_complaint_buttons, do_complaint, \
+        do_complaint_warning, do_exception_complaint, \
+        do_complaint_error_details
 from sutekh.io.ZipFileWrapper import ZipFileWrapper
 from sutekh.io.WwFile import WwFile
-from sutekh.core.SutekhObjects import aObjectList, PhysicalCardSet
+from sutekh.core.SutekhObjects import TABLE_LIST, PhysicalCardSet
 from sutekh.SutekhUtility import refresh_tables, read_rulings, \
-        read_white_wolf_list
+        read_white_wolf_list, get_cs_id_name_table
 
 
 def read_cardlist(oCardList, oProgressDialog, oLogHandler):
@@ -29,23 +31,36 @@ def read_cardlist(oCardList, oProgressDialog, oLogHandler):
     read_white_wolf_list(oCardList, oLogHandler)
     oProgressDialog.set_complete()
 
-def read_ww_rulings(oRulings, oProgressDialog, oLogHandler):
+
+def read_extra(oExtraFile, oProgressDialog, oLogHandler):
+    """Read extra card info into the database"""
+    oProgressDialog.set_description("Reading Extra Card Info")
+    oProgressDialog.show()
+    read_white_wolf_list(oExtraFile, oLogHandler)
+    oProgressDialog.set_complete()
+
+
+def read_ww_rulings(aRulingFiles, oProgressDialog, oLogHandler):
     """Read the WW Rulings file into the database"""
     oProgressDialog.set_description("Reading White Wolf Rulings")
     oProgressDialog.reset()
-    read_rulings(oRulings, oLogHandler)
+    read_rulings(aRulingFiles, oLogHandler)
     oProgressDialog.set_complete()
 
-def read_ww_lists_into_db(aCLFile, oRulingsFile, oProgressDialog):
+
+def read_ww_lists_into_db(aCLFile, oExtraFile, aRulingsFiles, oProgressDialog):
     """Read WW card list and possibly rulings into the given database"""
-    refresh_tables(aObjectList, sqlhub.processConnection)
+    refresh_tables(TABLE_LIST, sqlhub.processConnection)
     oProgressDialog.reset()
     # WhiteWolf Parser uses sqlhub connection
     oLogHandler = SutekhHTMLLogHandler()
     oLogHandler.set_dialog(oProgressDialog)
     read_cardlist(aCLFile, oProgressDialog, oLogHandler)
-    if oRulingsFile:
-        read_ww_rulings(oRulingsFile, oProgressDialog,  oLogHandler)
+    if oExtraFile:
+        read_extra([oExtraFile], oProgressDialog, oLogHandler)
+    if aRulingsFiles:
+        read_ww_rulings(aRulingsFiles, oProgressDialog,  oLogHandler)
+
 
 def copy_to_new_db(oOldConn, oTempConn, oWin, oProgressDialog, oLogHandler):
     """Copy card collection and card sets to a new abstract card db."""
@@ -55,7 +70,7 @@ def copy_to_new_db(oOldConn, oTempConn, oWin, oProgressDialog, oLogHandler):
             oWin.cardLookup, oLogHandler)
     oProgressDialog.set_complete()
     if not bOK:
-        sMesg = "\n".join (["There was a problem copying your collection"
+        sMesg = "\n".join(["There was a problem copying your collection"
             " to the new database"] + aErrors +
                 ["Attempt to Continue Anyway (This is most probably"
                     " very dangerous)?"])
@@ -66,28 +81,36 @@ def copy_to_new_db(oOldConn, oTempConn, oWin, oProgressDialog, oLogHandler):
             return False
     return True
 
+
 def initialize_db(oParent):
-    """Initailize the database if it doesn't exist"""
+    """Initialise the database if it doesn't exist"""
     iRes = do_complaint_buttons("The database doesn't seem to be properly"
             " initialised",
             gtk.MESSAGE_ERROR, (gtk.STOCK_QUIT, gtk.RESPONSE_CLOSE,
                 "Initialise database with cardlist and rulings?", 1))
-    # pylint: disable-msg=W0612
-    # sIgnore is ignored here
     if iRes != 1:
         return False
     else:
-        aCLFile, oRulingsFile, sIgnore = _get_names(oParent)
+        aCLFile, oExtraFile, aRulingsFiles, _sIgnore = _get_names(oParent,
+                True)
         if aCLFile is not None:
             oProgressDialog = ProgressDialog()
-            read_ww_lists_into_db(aCLFile, oRulingsFile, oProgressDialog)
-            oProgressDialog.destroy()
+            try:
+                read_ww_lists_into_db(aCLFile, oExtraFile, aRulingsFiles,
+                        oProgressDialog)
+                oProgressDialog.destroy()
+            except IOError, oErr:
+                do_exception_complaint("Failed to read cardlists.\n\n%s\n"
+                        "Aborting import." % oErr)
+                oProgressDialog.destroy()
+                return False
         else:
             return False
     # Create the Physical Card Collection card set
     PhysicalCardSet(name='My Collection', parent=None)
     return True
 
+
 def save_backup(sBackupFile, oProgressDialog):
     """Save a backup file, showing a progress dialog"""
     oLogHandler = SutekhCountLogHandler()
@@ -98,11 +121,13 @@ def save_backup(sBackupFile, oProgressDialog):
     oFile.do_dump_all_to_zip(oLogHandler)
     oProgressDialog.set_complete()
 
-def _get_names(oWin):
+
+def _get_names(oWin, bDisableBackup):
     """Get the names from the user, and ready them for the import."""
-    oWWFilesDialog = WWFilesDialog(oWin)
+    oWWFilesDialog = WWFilesDialog(oWin, bDisableBackup)
     oWWFilesDialog.run()
-    (sCLFileName, bCLIsUrl, sRulingsFileName, bRulingsIsUrl,
+    (sCLFileName, bCLIsUrl, sExtraFileName, bExtraIsUrl,
+            sRulingsFileName, bRulingsIsUrl,
             sBackupFile) = oWWFilesDialog.get_names()
     oWWFilesDialog.destroy()
     if sCLFileName is not None:
@@ -112,18 +137,31 @@ def _get_names(oWin):
             aCLFile = [WwFile(sCLFileName, bUrl=bCLIsUrl)]
     else:
         aCLFile = None
-    if sRulingsFileName is not None:
-        oRulingsFile = WwFile(sRulingsFileName, bUrl=bRulingsIsUrl)
+    if sExtraFileName is not None:
+        oExtraFile = WwFile(sExtraFileName, bUrl=bExtraIsUrl)
     else:
-        oRulingsFile = None
-    return aCLFile, oRulingsFile, sBackupFile
+        oExtraFile = None
+    if sRulingsFileName is not None:
+        if isinstance(sRulingsFileName, list):
+            aRulingsFiles = [WwFile(x, bUrl=bRulingsIsUrl) for x in
+                    sRulingsFileName]
+        else:
+            aRulingsFiles = [WwFile(sRulingsFileName, bUrl=bRulingsIsUrl)]
+    else:
+        aRulingsFiles = None
+    return aCLFile, oExtraFile, aRulingsFiles, sBackupFile
+
 
 def refresh_ww_card_list(oWin):
     """Handle grunt work of refreshing the card lists"""
+    # pylint: disable-msg=R0914
+    # We're juggling lots of different bits of start, so we use a lot
+    # of variables
     aEditable = oWin.get_editable_panes()
-    aCLFile, oRulingsFile, sBackupFile = _get_names(oWin)
+    dOldMap = get_cs_id_name_table()
+    aCLFile, oExtraFile, oRulingsFile, sBackupFile = _get_names(oWin, False)
     if not aCLFile:
-        return False # Nothing happened
+        return False  # Nothing happened
     oProgressDialog = ProgressDialog()
     if sBackupFile is not None:
         # pylint: disable-msg=W0703
@@ -131,13 +169,21 @@ def refresh_ww_card_list(oWin):
         try:
             save_backup(sBackupFile, oProgressDialog)
         except Exception, oErr:
-            do_complaint_error("Failed to write backup.\n\n%s\n"
+            do_exception_complaint("Failed to write backup.\n\n%s\n"
                     "Not touching the database further." % oErr)
             return False
     oOldConn = sqlhub.processConnection
     oTempConn = connectionForURI("sqlite:///:memory:")
     sqlhub.processConnection = oTempConn
-    read_ww_lists_into_db(aCLFile, oRulingsFile, oProgressDialog)
+    try:
+        read_ww_lists_into_db(aCLFile, oExtraFile, oRulingsFile,
+            oProgressDialog)
+    except IOError, oErr:
+        # Failed to read one of the card lists, so celan up and abort
+        do_exception_complaint("Failed to read cardlists.\n\n%s\n"
+                    "Aborting import." % oErr)
+        oProgressDialog.destroy()
+        return False
     # Refresh abstract card view for card lookups
     oLogHandler = SutekhCountLogHandler()
     oLogHandler.set_dialog(oProgressDialog)
@@ -145,30 +191,32 @@ def refresh_ww_card_list(oWin):
     if not copy_to_new_db(oOldConn, oTempConn, oWin, oProgressDialog,
             oLogHandler):
         oProgressDialog.destroy()
-        return True # Force refresh
+        return True  # Force refresh
     # OK, update complete, copy back from oTempConn
     sqlhub.processConnection = oOldConn
-    oWin.clear_cache() # Don't hold old copies
+    oWin.clear_cache()  # Don't hold old copies
     oProgressDialog.set_description("Finalizing import")
     oProgressDialog.reset()
     oProgressDialog.show()
     (bOK, aErrors) = create_final_copy(oTempConn, oLogHandler)
     if not bOK:
-        sMesg = "There was a problem updating the database\n"
-        for sStr in aErrors:
-            sMesg += sStr + "\n"
-        sMesg += "Your database may be in an inconsistent state -" \
+        sMesg = "There was a problem updating the database\n" \
+                "Your database may be in an inconsistent state -" \
                 " sorry"
-        do_complaint_error(sMesg)
+        logging.warn('\n'.join([sMesg] + aErrors))
+        do_complaint_error_details(sMesg, "\n".join(aErrors))
     else:
         sMesg = "Import Completed\n"
         sMesg += "Everything seems to have gone OK"
         do_complaint(sMesg, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, True)
     oProgressDialog.destroy()
+    dNewMap = get_cs_id_name_table()
+    oWin.config_file.fix_profile_mapping(dOldMap, dNewMap)
     oWin.update_to_new_db()
     oWin.restore_editable_panes(aEditable)
     return True
 
+
 def do_db_upgrade(aLowerTables, aHigherTables):
     """Attempt to upgrade the database"""
     if len(aHigherTables) > 0:
@@ -208,16 +256,17 @@ def do_db_upgrade(aLowerTables, aHigherTables):
                 sqlhub.processConnection = oTempConn
                 return True
         else:
-            sMesg = '\n'.join(["Unable to create memory copy!"] +
-                    aMessages + ["Upgrade Failed."])
-            do_complaint_error(sMesg)
+            sMesg = "Unable to create memory copy!\nUpgrade Failed."
+            logging.warn('\n'.join([sMesg] + aMessages))
+            do_complaint_error_details(sMesg, "\n".join(aMessages))
     except UnknownVersion, oErr:
         oProgressDialog.destroy()
-        do_complaint_error("Upgrade Failed. " + str(oErr))
+        do_exception_complaint("Upgrade Failed. %s" % oErr)
     return False
 
+
 def _do_commit_db(oLogHandler, oTempConn):
-    """Handle commiting the memory copy to the actual DB"""
+    """Handle committing the memory copy to the actual DB"""
     oProgressDialog = ProgressDialog()
     oProgressDialog.set_description("Commiting changes")
     oLogHandler.set_dialog(oProgressDialog)
@@ -225,7 +274,7 @@ def _do_commit_db(oLogHandler, oTempConn):
     oProgressDialog.destroy()
     if bOK:
         sMesg = "Changes Commited\n"
-        if len(aMessages)>0:
+        if len(aMessages) > 0:
             sMesg += '\n'.join(["Messages reported are:"] +
                     aMessages)
         else:
@@ -233,9 +282,8 @@ def _do_commit_db(oLogHandler, oTempConn):
         do_complaint(sMesg, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, True)
         return True
     else:
-        sMesg = '\n'.join(["Unable to commit updated database!"] +
-                aMessages +
-                ["Upgrade Failed.", "Your database may be"
-                    " in an inconsistent state."])
-        do_complaint_error(sMesg)
+        sMesg = "Unable to commit updated database!\nUpgrade Failed.\n" \
+                "Your database may be in an inconsistent state."
+        logging.warn('\n'.join([sMesg] + aMessages))
+        do_complaint_error_details(sMesg, "\n".join(aMessages))
         return False
