@@ -13,10 +13,12 @@
 
 import gtk
 
+
 def _changed_dir(oFileChooser, oParentWin):
     """Update parent's working dir when the folder changes."""
     oParentWin.set_working_dir(oFileChooser.get_current_folder())
 
+
 def add_filter(oFileChooser, sFilterName, aFilterPatterns):
     """Add  filter to the widget, using the list of patterns in
        aFilterPatterns"""
@@ -28,6 +30,7 @@ def add_filter(oFileChooser, sFilterName, aFilterPatterns):
     oFileChooser.set_filter(oFilter)
     return oFilter
 
+
 def _mapped(oFileWidget, oParent):
     """Update the dialogs working dir when the widget is shown.
 
@@ -38,6 +41,7 @@ def _mapped(oFileWidget, oParent):
     if sWorkingDir:
         oFileWidget.set_current_folder(sWorkingDir)
 
+
 class SutekhFileDialog(gtk.FileChooserDialog):
     # pylint: disable-msg=R0904
     # gtk widget, so has many public methods
@@ -65,6 +69,7 @@ class SutekhFileDialog(gtk.FileChooserDialog):
         """Set the filter to be the default all files filter"""
         self.set_filter(self._oAllFilter)
 
+
 class SutekhFileWidget(gtk.FileChooserWidget):
     # pylint: disable-msg=R0904
     # gtk widget, so has many public methods
@@ -88,6 +93,7 @@ class SutekhFileWidget(gtk.FileChooserWidget):
         """Set the filter to be the default all files filter"""
         self.set_filter(self._oAllFilter)
 
+
 class SutekhFileButton(gtk.FileChooserButton):
     # pylint: disable-msg=R0904
     # gtk widget, so has many public methods
@@ -112,6 +118,7 @@ class SutekhFileButton(gtk.FileChooserButton):
         """Set the filter to be the default all files filter"""
         self.oDialog.default_filter()
 
+
 class SimpleFileDialog(SutekhFileDialog):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -125,9 +132,7 @@ class SimpleFileDialog(SutekhFileDialog):
         self.set_select_multiple(False)
         self.sName = None
 
-    # pylint: disable-msg=W0613
-    # oWidget required by function signature
-    def button_response(self, oWidget, iResponse):
+    def button_response(self, _oWidget, iResponse):
         """Handle button press events"""
         if iResponse == gtk.RESPONSE_OK:
             self.sName = self.get_filename()
@@ -137,6 +142,7 @@ class SimpleFileDialog(SutekhFileDialog):
         """Return the name to the caller."""
         return self.sName
 
+
 class ImportDialog(SimpleFileDialog):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -145,6 +151,7 @@ class ImportDialog(SimpleFileDialog):
         super(ImportDialog, self).__init__(oParent, sTitle,
                 gtk.FILE_CHOOSER_ACTION_OPEN)
 
+
 class ExportDialog(SimpleFileDialog):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -155,3 +162,12 @@ class ExportDialog(SimpleFileDialog):
         self.set_do_overwrite_confirmation(True)
         if sDefaultFileName:
             self.set_current_name(sDefaultFileName)
+
+
+class ZipFileDialog(SimpleFileDialog):
+    # pylint: disable-msg=R0904
+    # gtk.Widget, so many public methods
+    """Prompt the user for a zip file name"""
+    def __init__(self, oParent, sTitle, oAction):
+        super(ZipFileDialog, self).__init__(oParent, sTitle, oAction)
+        self.add_filter_with_pattern('Zip Files', ['*.zip', '*.ZIP'])
