@@ -9,6 +9,7 @@
 
 import gtk
 
+
 class SutekhMenu(gtk.MenuBar):
     # pylint: disable-msg=R0904
     # R0904 - gtk.Widget, so many public methods
@@ -57,7 +58,8 @@ class SutekhMenu(gtk.MenuBar):
            Create a menu item, connect it to fAction (if not None), and
            add an accelerator if specified."""
         oMenuItem = gtk.MenuItem(sName)
-        oMenu.add(oMenuItem)
+        if oMenu is not None:
+            oMenu.add(oMenuItem)
         if fAction:
             oMenuItem.connect('activate', fAction)
         if sAccelKey:
@@ -104,14 +106,36 @@ class SutekhMenu(gtk.MenuBar):
         oMenuItem = self.create_menu_item_with_submenu(oMenu, sName)
         return oMenuItem.get_submenu()
 
-
-    def create_plugins_menu(self, sName, oPluginWindow):
-        """Create a menu holding the plugins for oPluginWindow."""
-        oMenuItem = self.create_menu_item_with_submenu(self, sName)
+    def add_plugins_to_menus(self, oPluginWindow):
+        """Add the plugins for oPluginWindow to the current menus."""
+        oMenuItem = self.create_menu_item_with_submenu(self, 'Other')
         oMenu = oMenuItem.get_submenu()
         # Add plugins
         for oPlugin in oPluginWindow.plugins:
             oPlugin.add_to_menu(self._dMenus, oMenu)
         if len(oMenu.get_children()) == 0:
-            oMenuItem.set_sensitive(False)
+            self.remove(oMenuItem)
+            del self._dMenus['Other']
+        else:
+            self.sort_menu(oMenu)
 
+    @staticmethod
+    def sort_menu(oMenu):
+        """Sort the entries of a sub-menu by the menu text"""
+        def get_name(oWidget):
+            """Get the text label from an menu item by descending the
+               children"""
+            if isinstance(oWidget, gtk.Label):
+                return oWidget.get_text()
+            if not hasattr(oWidget, 'get_children'):
+                return None
+            for oChild in oWidget.get_children():
+                sName = get_name(oChild)
+                if sName:
+                    return sName
+            return None
+
+        aItems = [(get_name(x), x) for x in oMenu.get_children()]
+        aItems.sort()
+        for iPos, (_sText, oItem) in enumerate(aItems):
+            oMenu.reorder_child(oItem, iPos)
