@@ -12,10 +12,12 @@
 
 import zipfile
 import datetime
+from StringIO import StringIO
 from logging import Logger
 from sqlobject import sqlhub, SQLObjectNotFound
-from sutekh.core.SutekhObjects import PhysicalCardSet, aPhysicalSetList
+from sutekh.core.SutekhObjects import PhysicalCardSet, PHYSICAL_SET_LIST
 from sutekh.core.CardLookup import DEFAULT_LOOKUP
+from sutekh.core.CardSetHolder import CachedCardSetHolder, CardSetWrapper
 from sutekh.SutekhUtility import refresh_tables
 from sutekh.io.PhysicalCardParser import PhysicalCardParser
 from sutekh.io.PhysicalCardSetParser import PhysicalCardSetParser
@@ -23,6 +25,15 @@ from sutekh.io.AbstractCardSetParser import AbstractCardSetParser
 from sutekh.io.PhysicalCardSetWriter import PhysicalCardSetWriter
 from sutekh.io.IdentifyXMLFile import IdentifyXMLFile
 
+
+def _parse_string(oParser, sIn, oHolder):
+    """Utitlity function for reading zip files.
+
+       Allows oParser.parse to be called on a string."""
+    oFile = StringIO(sIn)
+    oParser.parse(oFile, oHolder)
+
+
 class ZipFileWrapper(object):
     """The zip file wrapper.
 
@@ -46,22 +57,25 @@ class ZipFileWrapper(object):
         self.oZip.close()
         self.oZip = None
 
-    def write_all_pcs_to_zip(self, oLogger):
-        """Add all the physical card sets to the zip file"""
+    def write_pcs_list_to_zip(self, aPCSList, oLogger):
+        """Write the given list of card sets to the zip file"""
         bClose = False
         tTime = datetime.datetime.now().timetuple()
         if self.oZip is None:
             self.__open_zip_for_write()
             bClose = True
-        oPhysicalCardSets = PhysicalCardSet.select()
         aList = []
-        for oPCSet in oPhysicalCardSets:
+        for oPCSet in aPCSList:
             sZName = oPCSet.name
             oWriter = PhysicalCardSetWriter()
-            oString = oWriter.gen_xml_string(sZName)
+            oHolder = CardSetWrapper(oPCSet)
+            oFile = StringIO()
+            oWriter.write(oFile, oHolder)
+            oString = oFile.getvalue()
+            oFile.close()
             sZName = sZName.replace(" ", "_")
             sZName = sZName.replace("/", "_")
-            sZipName = 'pcs_%s.xml' % sZName
+            sZipName = '%s.xml' % sZName
             sZipName = sZipName.encode('ascii', 'xmlcharrefreplace')
             aList.append(sZipName)
             # ZipInfo will just use the 1st 6 fields in tTime
@@ -69,6 +83,7 @@ class ZipFileWrapper(object):
             # Set permissions on the created file - see issue 3394 on the
             # python bugtracker. Docs say this is safe on all platforms
             oInfoObj.external_attr = 0600 << 16L
+            oInfoObj.compress_type = zipfile.ZIP_DEFLATED
             self.oZip.writestr(oInfoObj, oString)
             oLogger.info('PCS: %s written', oPCSet.name)
         if bClose:
@@ -85,7 +100,7 @@ class ZipFileWrapper(object):
         oLogger = Logger('Restore zip file')
         if oLogHandler is not None:
             oLogger.addHandler(oLogHandler)
-            if hasattr(oLogHandler,'set_total'):
+            if hasattr(oLogHandler, 'set_total'):
                 oLogHandler.set_total(len(self.oZip.infolist()))
         # We do this so we can accomodate user created zipfiles,
         # that don't nessecarily have the ordering we want
@@ -94,7 +109,7 @@ class ZipFileWrapper(object):
         # PhysicalCard list
         for oItem in self.oZip.infolist():
             oData = self.oZip.read(oItem.filename)
-            oIdParser.parse_string(oData)
+            _parse_string(oIdParser, oData, None)
             if (oIdParser.type == 'PhysicalCard' or \
                     oIdParser.type == 'PhysicalCardSet') and not \
                     bTablesRefreshed:
@@ -103,7 +118,7 @@ class ZipFileWrapper(object):
                 # hopefully this is safe to do
                 # if we fail, the database will be in an inconsitent state,
                 # but that's going to be true anyway
-                refresh_tables(aPhysicalSetList, sqlhub.processConnection)
+                refresh_tables(PHYSICAL_SET_LIST, sqlhub.processConnection)
                 bTablesRefreshed = True
             if oIdParser.type == 'PhysicalCard':
                 bOldStyle = True
@@ -128,13 +143,14 @@ class ZipFileWrapper(object):
         for oItem in aList:
             bReparent = False
             oData = self.oZip.read(oItem.filename)
-            oIdParser.parse_string(oData)
+            _parse_string(oIdParser, oData, None)
             if oIdParser.type == 'PhysicalCardSet':
                 # We check whether the parent has been read already
                 if bOldStyle:
                     # We need to reparent this card set
                     try:
-                        oParent = PhysicalCardSet.selectBy(
+                        # card set holder will handle setting the parent
+                        PhysicalCardSet.selectBy(
                                 name='My Collection').getOne()
                         bReparent = True
                         oParser = PhysicalCardSetParser()
@@ -153,15 +169,15 @@ class ZipFileWrapper(object):
                 oParser = PhysicalCardParser()
             else:
                 continue
-            self._aWarnings.extend(oParser.parse_string(oData, oCardLookup,
-                True, dLookupCache))
-            oLogger.info('%s %s read', oIdParser.type, oItem.filename)
+            oHolder = CachedCardSetHolder()
+            _parse_string(oParser, oData, oHolder)
             if bReparent:
                 # pylint: disable-msg=E1103
                 # SQLObject confuses pylint
-                oPCS = PhysicalCardSet.selectBy(name=oIdParser.name).getOne()
-                oPCS.parent = oParent
-                oPCS.syncUpdate()
+                oHolder.parent = 'My Collection'
+            oHolder.create_pcs(oCardLookup, dLookupCache)
+            self._aWarnings.extend(oHolder.get_warnings())
+            oLogger.info('%s %s read', oIdParser.type, oItem.filename)
         if len(aToRead) == len(aList):
             # We were unable to read any items this loop, so we fail
             raise IOError('Card sets with unstatisfiable parents %s' %
@@ -172,14 +188,23 @@ class ZipFileWrapper(object):
 
     def do_dump_all_to_zip(self, oLogHandler=None):
         """Dump all the database contents to the zip file"""
+        aPhysicalCardSets = PhysicalCardSet.select()
+        return self.do_dump_list_to_zip(aPhysicalCardSets, oLogHandler)
+
+    def do_dump_list_to_zip(self, aCSList, oLogHandler=None):
+        """Handle dumping a list of cards to the zip file with log fiddling"""
         self.__open_zip_for_write()
-        oLogger = Logger('Restore zip file')
+        oLogger = Logger('Write zip file')
         if oLogHandler is not None:
             oLogger.addHandler(oLogHandler)
-            if hasattr(oLogHandler,'set_total'):
-                iTotal = PhysicalCardSet.select().count()
-                oLogHandler.set_total(iTotal)
-        aPCSList = self.write_all_pcs_to_zip(oLogger)
+            if hasattr(oLogHandler, 'set_total'):
+                if hasattr(aCSList, 'count'):
+                    # Handle case we have a select result list
+                    iTotal = aCSList.count()
+                    oLogHandler.set_total(iTotal)
+                else:
+                    oLogHandler.set_total(len(aCSList))
+        aPCSList = self.write_pcs_list_to_zip(aCSList, oLogger)
         self.__close_zip()
         return aPCSList
 
@@ -190,7 +215,7 @@ class ZipFileWrapper(object):
         oIdParser = IdentifyXMLFile()
         for oItem in self.oZip.infolist():
             oData = self.oZip.read(oItem.filename)
-            oIdParser.parse_string(oData)
+            _parse_string(oIdParser, oData, None)
             if oIdParser.type == 'PhysicalCardSet':
                 dCardSets[oIdParser.name] = (oItem.filename,
                         oIdParser.parent_exists, oIdParser.parent)
@@ -202,15 +227,14 @@ class ZipFileWrapper(object):
         self.__open_zip_for_read()
         oIdParser = IdentifyXMLFile()
         oData = self.oZip.read(sFilename)
-        oIdParser.parse_string(oData)
-        oParser = None
+        _parse_string(oIdParser, oData, None)
+        oHolder = None
         if oIdParser.type == 'PhysicalCardSet':
             oParser = PhysicalCardSetParser()
-            oParser.string_to_holder(oData)
+            oHolder = CachedCardSetHolder()
+            _parse_string(oParser, oData, oHolder)
         self.__close_zip()
-        if oParser:
-            return oParser.oCS
-        return None
+        return oHolder
 
     def get_warnings(self):
         """Get any warnings from the process"""
