@@ -10,10 +10,29 @@ from sutekh.tests.TestCore import SutekhTest
 from sutekh.tests.io import test_WhiteWolfParser
 from sutekh.core.SutekhObjects import IAbstractCard, IPhysicalCard, \
         IPhysicalCardSet, PhysicalCardSet
+from sutekh.core.CardSetHolder import CardSetHolder
 from sutekh.io.PhysicalCardParser import PhysicalCardParser
-from sutekh.io.IdentifyXMLFile import IdentifyXMLFile
 from sutekh.io.XmlFileHandling import PhysicalCardXmlFile
-import unittest, os
+from sutekh.tests.core.test_PhysicalCardSet import ABSTRACT_CARDS
+import unittest
+import os
+from StringIO import StringIO
+
+LAST_WRITER_VERSION = "1.0"
+
+
+def make_example_pcxml():
+    """Create the example XML File"""
+    # pylint: disable-msg=E1101
+    # E1101: SQLObject + PyProtocols magic confuses pylint
+    oAC = IAbstractCard(ABSTRACT_CARDS[0])
+    oPC = IPhysicalCard((oAC, None))
+
+    sExample = '<cards sutekh_xml_version="%s"><card count="1" ' \
+            'expansion="None Specified" id="%d" name="%s" /></cards>' \
+            % (LAST_WRITER_VERSION, oPC.id, oAC.name)
+    return sExample
+
 
 class PhysicalCardTests(SutekhTest):
     """class for the PhysicalCard tests"""
@@ -21,42 +40,43 @@ class PhysicalCardTests(SutekhTest):
 
     def test_physical(self):
         """Test physical card handling"""
-        # pylint: disable-msg=E1101, R0915, R0914
+        # pylint: disable-msg=E1101
         # E1101: SQLObject + PyProtocols magic confuses pylint
-        # R0915, R0914: Want a long, sequential test case to minimise
-        # repeated setups, so it has lots of lines + variables
 
-        aAbstractCards = ['.44 magnum', 'ak-47', 'abbot', 'abebe', 'abombwe']
         # test IO
-
-        oAC = IAbstractCard(aAbstractCards[0])
-        oPC = IPhysicalCard((oAC, None))
-        sLastWriterVersion = "1.0"
-
-        sExample = '<cards sutekh_xml_version="%s"><card count="1" ' \
-                   'expansion="None Specified" id="%d" name="%s" /></cards>' \
-                   % (sLastWriterVersion, oPC.id, oAC.name)
-
+        sExample = make_example_pcxml()
         oParser = PhysicalCardParser()
-        oParser.parse_string(sExample)
+        oHolder = CardSetHolder()
+        oParser.parse(StringIO(sExample), oHolder)
+        oHolder.create_pcs()
 
         oMyCollection = IPhysicalCardSet("My Collection")
         assert(len(oMyCollection.cards) == 1)
         PhysicalCardSet.delete(oMyCollection.id)
 
-        sTempFileName =  self._create_tmp_file()
+        sTempFileName = self._create_tmp_file()
         fOut = open(sTempFileName, 'w')
         fOut.write(sExample)
         fOut.close()
 
-        oIdFile = IdentifyXMLFile()
-        oIdFile.id_file(sTempFileName)
-        self.assertEqual(oIdFile.type, 'PhysicalCard')
-
         fIn = open(sTempFileName, 'rU')
-        oParser.parse(fIn)
+        oHolder = CardSetHolder()
+        oParser.parse(fIn, oHolder)
         fIn.close()
+        oHolder.create_pcs()
 
+        # Test incorrect input
+        oHolder = CardSetHolder()
+        self.assertRaises(IOError, oParser.parse, StringIO(
+            '<ccaardd sutekh_xml_version="1.0"><card count="1" ' \
+            'expansion="None Specified" id="12" name="Abbot" /></ccaardd>'),
+            oHolder)
+        self.assertRaises(IOError, oParser.parse, StringIO(
+            '<cards sutekh_xml_version="5.0"><card count="1" ' \
+            'expansion="None Specified" id="12" name="Abbot" /></cards>'),
+            oHolder)
+
+        # Check read results
         oMyCollection = IPhysicalCardSet("My Collection")
         assert(len(oMyCollection.cards) == 1)
         PhysicalCardSet.delete(oMyCollection.id)
